/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QColor>
#include <QPainter>
#include <QDebug>
#include <QMessageBox>
#include <QtGlobal>
#include <math.h>
#include "practico1.h"

Practico1::Practico1(QWidget * parent) : QMainWindow(parent)
{
	// We set the main window
	mainWindow.setupUi(this);

	// Then the image
	imagePosition = QPoint(10,70);
	imageWidth = 740;
	imageHeight = 540;
	image = QImage(imageWidth, imageHeight, QImage::Format_RGB32);

	// The center of the image coordinates
	centerX = imageWidth / 2;
	centerY = imageHeight / 2;

	// Some flags
	mouseInputExists = false;
	firstPoint = QPoint(0,0);
	secondPoint = QPoint(0,0);

	// Seed
	seed = 47;
}


void Practico1::paintEvent(QPaintEvent * )
{
	QPainter painter(this);
	painter.drawImage(imagePosition,image);

	exampleDDALines();
	exampleMidPointLines();

	circleDDA(QPoint(0, 0), 128, QColor("red"));
	circleMidPoint(QPoint(0, 0), 128, QColor("green"));
	circleBresenham(QPoint(0, 0), 128, QColor("blue"));

	circleDDA(QPoint(0, -128), 128, QColor("red"));
	circleMidPoint(QPoint(-128, 128), 128, QColor("green"));
	circleBresenham(QPoint(128, 128), 128, QColor("blue"));
	scanlinedTriangles(QPoint(-350,-250), QPoint(-250,0), QPoint(-300, 200), true, false);
	scanlinedTriangles(QPoint(300,200), QPoint(200,0), QPoint(180, 250), false, false);
	scanlinedTriangles(QPoint(-250,-250), QPoint(-150,0), QPoint(-200, 200), true, true);
}

void Practico1::lineDDA(QPointF begin, QPointF end, QColor color)
{
	QPointF swap;
	double m = 0.0;
	bool inverseDelta = false;
	bool negativeDeltaX = false;

	// First we check if we have a vertical line
	if(begin.x() == end.x())
	{
		// We check in which direction we are drawing
		if(begin.y() > end.y())
		{
			// We swap the points
			swap = begin;
			begin = end;
			end = swap;
		}
		// We finally draw
		for(int i = begin.y(); i<=end.y(); i++)
			image.setPixel(begin.x() + centerX, i + centerY, color.rgb());
		return;
	}

	// We check for an inverse delta
	inverseDelta =  abs(end.x() - begin.x()) < abs(end.y() - begin.y());
	if(inverseDelta)
	{
		swap = begin;
		begin.rx() = swap.y();
		begin.ry() = swap.x();

		swap = end;
		end.rx() = swap.y();
		end.ry() = swap.x();
	}

	// We check for a negative delta X
	if(end.x() < begin.x())
		negativeDeltaX = true;

	m = (end.y() - begin.y()) / (end.x() - begin.x());


	while(begin.x() != end.x())
	{
		if(inverseDelta)
			image.setPixel((int)begin.y() + centerX, (int)begin.x() + centerY,
			               color.rgb());
		else
			image.setPixel((int)begin.x() + centerX, (int)begin.y() + centerY,
			               color.rgb());

		if(negativeDeltaX)
		{
			begin.rx()--;
			begin.ry() -= m;
		}
		else
		{
			begin.rx()++;
			begin.ry() += m;
		}
	}
}

void Practico1::lineMidPoint(QPoint begin, QPoint end, QColor color)
{
	QPoint swap(0,0);
	QPoint point(0,0);
	QPoint deltaPoint(0,0);
	QPoint deltaPoint2(0,0);
	int e = 0;
	int slope = 1;
	bool steep = false;

	// First we check if we have a vertical line
	if(begin.x() == end.x())
	{
		// We check in which direction we are drawing
		if(begin.y() > end.y())
		{
			// We swap the points
			swap = begin;
			begin = end;
			end = swap;
		}

		for(int i = begin.y(); i<=end.y(); i++)
			image.setPixel(begin.x() + centerX, i + centerY, color.rgb());
		return;
	}

	// We check if we have a steep slope
	steep = abs(end.y() - begin.y()) > abs(end.x() - begin.x());
	if(steep)
	{
		swap = begin;
		begin.rx() = swap.y();
		begin.ry() = swap.x();

		swap = end;
		end.rx() = swap.y();
		end.ry() = swap.x();
	}

	// If we have a negative delta X, we swap the points
	if(begin.x() > end.x())
	{
		swap = begin;
		begin = end;
		end = swap;
	}

	deltaPoint.rx() = end.x() - begin.x();
	deltaPoint.ry() = end.y() - begin.y();

	// If we have negative slopes
	if(deltaPoint.y() < 0)
	{
		slope = -1;
		deltaPoint.ry() = -deltaPoint.y();
	}

	deltaPoint2 = 2 * deltaPoint;

	point.ry() = begin.y();
	e = deltaPoint2.y() - deltaPoint.x();

	for(point.rx() = begin.x(); point.x() < end.x(); point.rx()++)
	{
		if(steep)
			image.setPixel(point.y() + centerX, point.x() + centerY, color.rgb());
		else
			image.setPixel(point.x() + centerX, point.y() + centerY, color.rgb());

		e += deltaPoint2.y();
		if(e > 0)
		{
			e -= deltaPoint2.x();
			point.ry() += slope;
		}
	}

}

void Practico1::circleDDA(QPoint center, double radius, QColor color)
{
	int x = (int) radius;
	int y = 0;

	while(y < x)
	{
		image.setPixel(x + center.x() + centerX,
		               y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		                y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());

		y++;
		radius = radius - y / radius;
		x = (int) radius;
	}
}

void Practico1::circleMidPoint(QPoint center, double radius, QColor color)
{
	int x = (int) radius;
	int y = 0;
	double d = 5.0/4.0 - radius;

		image.setPixel(x + center.x() + centerX,
		               y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		                y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());

	while(y < x)
	{
		if(d < 0)
		{
			d = d + 2*y + 3;
			y++;
		}
		else
		{
			d = d + 2*y - 2*x +5;
			y++;
			x--;
		}
		image.setPixel(x + center.x() + centerX,
		               y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		                y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());
	}
}

void Practico1::circleBresenham(QPoint center, double radius, QColor color)
{
	int x = (int) radius;
	int y = 0;
	int e = 0;

	while(y < x)
	{
		image.setPixel(x + center.x() + centerX,
		               y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		                y + center.y() + centerY, color.rgb());
		image.setPixel(y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(-x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               -x + center.y() + centerY, color.rgb());
		image.setPixel(x + center.x() + centerX,
		               -y + center.y() + centerY, color.rgb());
		image.setPixel(-y + center.x() + centerX,
		               x + center.y() + centerY, color.rgb());

		e = e + 2*y +1;
		y++;
		if(2*e > 2*x-1)
		{
			x--;
			e = e - 2*x +1;
		}
	}
}

void Practico1::exampleDDALines()
{
	// The asterisk
	lineDDA(QPoint(0, 0), QPoint(128, 32), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(128, 64), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(128, 96), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(128, 128), QColor("green"));

	lineDDA(QPoint(0, 0), QPoint(96, 128), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(64, 128), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(32, 128), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(0, 128), QColor("blue"));

	lineDDA(QPoint(0, 0), QPoint(-32, 128), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(-64, 128), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(-96, 128), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(-128, 128), QColor("red"));

	lineDDA(QPoint(0, 0), QPoint(-128, 96), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(-128, 64), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(-128, 32), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(-128, 0), QColor("green"));

	lineDDA(QPoint(0, 0), QPoint(-128, -32), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(-128, -64), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(-128, -96), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(-128, -128), QColor("blue"));

	lineDDA(QPoint(0, 0), QPoint(-96, -128), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(-64, -128), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(-32, -128), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(0, -128), QColor("red"));

	lineDDA(QPoint(0, 0), QPoint(32, -128), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(64, -128), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(96, -128), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(128, -128), QColor("green"));

	lineDDA(QPoint(0, 0), QPoint(128, -96), QColor("blue"));
	lineDDA(QPoint(0, 0), QPoint(128, -64), QColor("red"));
	lineDDA(QPoint(0, 0), QPoint(128, -32), QColor("green"));
	lineDDA(QPoint(0, 0), QPoint(128, 0), QColor("blue"));

	// Crossed squares, turns in clockwise direction
	lineDDA(QPoint(0, 32), QPoint(32, 0), QColor("green"));
	lineDDA(QPoint(32, 0), QPoint(0, -32), QColor("green"));
	lineDDA(QPoint(0, -32), QPoint(-32, 0), QColor("green"));
	lineDDA(QPoint(-32, 0), QPoint(0, 32), QColor("green"));

	// Crossed squares, turns in anti clockwise direction
	lineDDA(QPoint(0, 64), QPoint(64, 0), QColor("red"));
	lineDDA(QPoint(64, 0), QPoint(0, -64), QColor("red"));
	lineDDA(QPoint(0, -64), QPoint(-64, 0), QColor("red"));
	lineDDA(QPoint(-64, 0), QPoint(0, 64), QColor("red"));

	// Blue can't be missing
	lineDDA(QPoint(0, 128), QPoint(128, 0), QColor("blue"));
	lineDDA(QPoint(128, 0), QPoint(0, -128), QColor("blue"));
	lineDDA(QPoint(0, -128), QPoint(-128, 0), QColor("blue"));
	lineDDA(QPoint(-128, 0), QPoint(0, 128), QColor("blue"));
}

void Practico1::exampleMidPointLines()
{

		// The asterisk
	lineMidPoint(QPoint(0, 0), QPoint(128, 32), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(128, 64), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(128, 96), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(128, 128), QColor("blue"));

	lineMidPoint(QPoint(0, 0), QPoint(96, 128), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(64, 128), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(32, 128), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(0, 128), QColor("red"));

	lineMidPoint(QPoint(0, 0), QPoint(-32, 128), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(-64, 128), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(-96, 128), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, 128), QColor("green"));

	lineMidPoint(QPoint(0, 0), QPoint(-128, 96), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, 64), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, 32), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, 0), QColor("blue"));

	lineMidPoint(QPoint(0, 0), QPoint(-128, -32), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, -64), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, -96), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(-128, -128), QColor("red"));

	lineMidPoint(QPoint(0, 0), QPoint(-96, -128), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(-64, -128), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(-32, -128), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(0, -128), QColor("green"));

	lineMidPoint(QPoint(0, 0), QPoint(32, -128), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(64, -128), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(96, -128), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(128, -128), QColor("blue"));

	lineMidPoint(QPoint(0, 0), QPoint(128, -96), QColor("red"));
	lineMidPoint(QPoint(0, 0), QPoint(128, -64), QColor("green"));
	lineMidPoint(QPoint(0, 0), QPoint(128, -32), QColor("blue"));
	lineMidPoint(QPoint(0, 0), QPoint(128, 0), QColor("red"));

	// Crossed squares, turns in clockwise direction
	lineMidPoint(QPoint(0, 16), QPoint(16, 0), QColor("red"));
	lineMidPoint(QPoint(16, 0), QPoint(0, -16), QColor("red"));
	lineMidPoint(QPoint(0, -16), QPoint(-16, 0), QColor("red"));
	lineMidPoint(QPoint(-16, 0), QPoint(0, 16), QColor("red"));

	// Crossed squares, turns in anti clockwise direction
	lineMidPoint(QPoint(0, 48), QPoint(48, 0), QColor("blue"));
	lineMidPoint(QPoint(48, 0), QPoint(0, -48), QColor("blue"));
	lineMidPoint(QPoint(0, -48), QPoint(-48, 0), QColor("blue"));
	lineMidPoint(QPoint(-48, 0), QPoint(0, 48), QColor("blue"));

	// Green can't be missing
	lineMidPoint(QPoint(0, 96), QPoint(96, 0), QColor("green"));
	lineMidPoint(QPoint(96, 0), QPoint(0, -96), QColor("green"));
	lineMidPoint(QPoint(0, -96), QPoint(-96, 0), QColor("green"));
	lineMidPoint(QPoint(-96, 0), QPoint(0, 96), QColor("green"));
}

void Practico1::scanlinedTriangles(QPoint first, QPoint second,
                                   QPoint third, bool palletize,
                                   bool randomize)
{
	int xMin(0), xMax(0);
	QColor minColor("black"), maxColor("black");
	double rm(0), gm(0), bm(0);
	double rb(0), gb(0), bb(0);


	ScanLineTable table(imageHeight);

	// I will use the DDA algorythm for the lines
	lineDDAScanLine(first, second, QColor(255, 0, 0), QColor(0, 255, 0),
	                table, palletize, randomize);
	lineDDAScanLine(second, third, QColor(0, 255, 0), QColor(0, 0, 255),
	                table, palletize, randomize);
	lineDDAScanLine(third, first, QColor(0, 0, 255), QColor(255, 0, 0),
	                table, palletize, randomize);

	for(int i = 0; i < imageHeight; i++)
	{
		table.getXMinMax(i, xMin, minColor, xMax, maxColor);

		// We calculate the color indexes
		if((xMax - xMin) != 0)
		{
			rm = (double)(maxColor.red() - minColor.red()) / (double)(xMax - xMin);
			gm = (double)(maxColor.green() - minColor.green()) / (double)(xMax - xMin);
			bm = (double)(maxColor.blue() - minColor.blue()) / (double)(xMax - xMin);
		}
		else
		{
			rm = 0.0;
			gm = 0.0;
			bm = 0.0;
		}

		rb = minColor.red() - rm*xMin;
		gb = minColor.green() - gm*xMin;
		bb = minColor.blue() - bm*xMin;

		for(int j=xMin; j <= xMax; j++)
		{
			if(palletize)
				image.setPixel(j, i, palletizeColor((rm*j)+rb,
				              (gm*j)+gb, (bm*j)+bb, randomize).rgb());
			else
				image.setPixel(j, i, QColor((rm*j)+rb,
				              (gm*j)+gb, (bm*j)+bb, randomize).rgb());
		}

	}
}

void Practico1::mousePressEvent(QMouseEvent * event)
{
	// We check that the left button has been pressed
	if(event->buttons() & Qt::LeftButton)
	{
// 		QString xCoord = QString(" %1 ").arg(event->x() - imagePosition.x());
// 		QString yCoord = QString(" %1 ").arg(event->y() - imagePosition.y());
// 		QMessageBox::information(this,tr("Práctico1"),
// 		                         tr("El punto donde se presionó es ") +
// 		                         xCoord + yCoord);
		if(!mouseInputExists)
		{
			firstPoint = event->pos() - imagePosition;
			mouseInputExists = true;
		}
		else
		{
			secondPoint = event->pos() - imagePosition;

			firstPoint -= QPoint(centerX, centerY);
			secondPoint -= QPoint(centerX, centerY);

			/// FIXME
			// Here we call the funtion in progress
			lineMidPoint(firstPoint, secondPoint, QColor("green"));

			double firstTerm = (secondPoint.x() - firstPoint.x());
			firstTerm *= firstTerm;
			double secondTerm = (secondPoint.y() - secondPoint.y());
			secondTerm *= secondTerm;
			int radius = (int)sqrt(firstTerm + secondTerm);

			circleMidPoint(firstPoint, radius, QColor("blue"));
			this->update();
			mouseInputExists = false;
		}
	}
}

void Practico1::lineDDAScanLine(QPointF begin, QPointF end,
                                QColor firstColor, QColor secondColor,
                                ScanLineTable & table, bool palletize,
                                bool randomize)
{
	QPointF swap;
	double m = 0.0;
	bool inverseDelta = false;
	bool negativeDeltaX = false;

	QColor tempColor;

	// First we check if we have a vertical line
	if(begin.x() == end.x())
	{
		// We check in which direction we are drawing
		if(begin.y() > end.y())
		{
			// We swap the points
			swap = begin;
			begin = end;
			end = swap;
		}
		// We finally draw
		for(int i = begin.y(); i<=end.y(); i++)
		{
			tempColor = interpolateColors(firstColor, secondColor,
			                              begin.y(), end.y(), i,
			                              palletize, randomize);

			image.setPixel(begin.x() + centerX, i + centerY, tempColor.rgb());
			table.addItem(begin.x() + centerX, i + centerY, tempColor.rgb());
		}
		return;
	}

	// We check for an inverse delta
	inverseDelta =  abs(end.x() - begin.x()) < abs(end.y() - begin.y());
	if(inverseDelta)
	{
		swap = begin;
		begin.rx() = swap.y();
		begin.ry() = swap.x();

		swap = end;
		end.rx() = swap.y();
		end.ry() = swap.x();
	}

	// We check for a negative delta X
	if(end.x() < begin.x())
		negativeDeltaX = true;

	// Hold the first x
	double firstX = begin.x();

	m = (end.y() - begin.y()) / (end.x() - begin.x());

	while(begin.x() != end.x())
	{
		if(inverseDelta)
		{
			tempColor = interpolateColors(firstColor, secondColor,
			                              firstX, end.x(), begin.x(),
			                              palletize, randomize);
			image.setPixel((int)begin.y() + centerX, (int)begin.x() + centerY,
			               tempColor.rgb());
			table.addItem((int)begin.y() + centerX, (int)begin.x() + centerY,
			              tempColor.rgb());
		}
		else
		{
			tempColor = interpolateColors(firstColor, secondColor,
			                              firstX, end.x(), begin.x(),
			                              palletize, randomize);
			image.setPixel((int)begin.x() + centerX, (int)begin.y() + centerY,
			               tempColor.rgb());
			table.addItem((int)begin.x() + centerX, (int)begin.y() + centerY,
			              tempColor.rgb());
		}

		if(negativeDeltaX)
		{
			begin.rx()--;
			begin.ry() -= m;
		}
		else
		{
			begin.rx()++;
			begin.ry() += m;
		}
	}
}

QColor Practico1::interpolateColors(QColor firstColor, QColor secondColor,
                                    int begin, int end, int position,
                                    bool palletize, bool randomize)
{
	double r(0), g(0), b(0);

	r = (double)(secondColor.red() - firstColor.red())/(double)(end - begin);
	r *= (position - begin);
	r += firstColor.red();

	g = (double)(secondColor.green() - firstColor.green())/(double)(end - begin);
	g *= (position - begin);
	g += firstColor.green();

	b = (double)(secondColor.blue() - firstColor.blue())/(double)(end - begin);
	b *= (position - begin);
	b += firstColor.blue();

	if(palletize)
		return palletizeColor((int)r, (int)g, (int)b, randomize);
	else
		return QColor((int)r, (int)g, (int)b);
}

QColor Practico1::palletizeColor(int r, int g, int b, bool randomize)
{
	int red = r;
	int green = g;
	int blue = b;

	if(r < 42)
		r = 0;
	else if((r >= 42) && (r < 84))
		r = 42;
	else if((r >= 84 ) && (r < 126))
		r = 84;
	else if((r >= 126) && (r < 168))
		r = 126;
	else if((r >= 168) && (r < 210))
		r = 168;
	else if((r >= 210) && (r < 252))
		r = 210;
	else
		r = 252;

	if(g < 36)
		g = 0;
	else if((g >= 36) && (g < 72))
		g = 36;
	else if((g <= 72 ) && (g < 108))
		g = 72;
	else if((g <= 108) && (g < 144))
		g = 108;
	else if((g <= 144) && (g < 180))
		g = 144;
	else if((g <= 180) && (g < 216))
		g = 180;
	else if((g <= 216) && (g < 252))
		g = 216;
	else
		g = 252;

	if(b < 42)
		b = 0;
	else if((b >= 42) && (b < 84))
		b = 42;
	else if((b <= 84 ) && (b < 126))
		b = 84;
	else if((b <= 126) && (b < 168))
		b = 126;
	else if((b <= 168) && (b < 210))
		b = 168;
	else if((b <= 210) && (b < 252))
		b = 210;
	else
		b = 252;

	if(randomize)
	{
		// Seed the random number generator
		qsrand(seed);

		double rand = (double)qrand() / (double)RAND_MAX;

		seed = qrand();

		if((rand >= (red - r)/42) && (r <= 210))
			r += 42;
		if((rand >= (green - g)/36) && (g <= 216))
			g += 36;
		if((rand >= (blue - b)/42) && (b <= 210))
			b += 42;
	}

	return QColor(r, g, b);
}

