/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PRACTICO1_H
#define PRACTICO1_H

#include <QObject>
#include <QPoint>
#include <QPointF>
#include <QColor>
#include <QMainWindow>
#include <QPaintEvent>
#include "ui/ui_mainwindow.h"
#include "scanlinetable.h"

//! Practico1 implements basic primitives and practico 4
/*!
	Practico1 is the main class of the first practice of
	Sistemas de Programación, UNS, year 2007.\n
	Note that the origin for all primitives is set in the middle of the image.

	Practico 4 is about colors, we will do different things with them.
*/

class Practico1: public QMainWindow
{
	Q_OBJECT

public:
	Practico1(QWidget * parent = 0);

	/// Here we draw the QImage
	void paintEvent(QPaintEvent * paintEvent);

	//!The DDA line algorythm.
	/*!
		\param begin is the first point.
		\param end is the second point.
		\param color define the color of the line.
	*/
	void lineDDA(QPointF begin, QPointF end, QColor color);

	//!The MidPoint line algorythm
	/*!
		\param begin is the first point.
		\param end is the second point.
		\param color define the color of the line.
	*/
	void lineMidPoint(QPoint begin, QPoint end, QColor color);

	//! The DDA circle algorythm
	/*!
		\param center is the first point.
		\param radius is the radius of the circle.
		\param color define the color of the circle.
	*/
	void circleDDA(QPoint center, double radius, QColor color);

	//! The MidPoint circle algorythm
	/*!
		\param center is the first point.
		\param radius is the radius of the circle.
		\param color define the color of the circle.
	*/
	void circleMidPoint(QPoint center, double radius, QColor color);

	//! The Bressenham circle algorythm
	/*!
		\param center is the first point.
		\param radius is the radius of the circle.
		\param color define the color of the circle.
	*/
	void circleBresenham(QPoint center, double radius, QColor color);

	//! DDA lines example
	/*!
		This example shows different lines done with the DDA algorythm.
	*/
	void exampleDDALines();

	//! Mid Point example
	/*!
		This example shows different lines done with the Mid Point algorythm.
	*/
	void exampleMidPointLines();

	//! Scanlined triangles
	/*!
		\param first is the first point.
		\param second is the second point.
		\param third is the third point.
	*/
	void scanlinedTriangles(QPoint first, QPoint second,
	                        QPoint third, bool palletize,
	                        bool randomize);

protected:
	/// Mouse action grabber
	/*!
		This method grabs all the mouse actions.
	*/
	void mousePressEvent(QMouseEvent *event);

private:

	/// The DDA line algorythm used by scanlinedTriangles
	/*!
	This is a Moco (tm), as I should use the lineDDA algorythm already
	implemented, and not re-implement it. Anyway, I want a quick-fix.
	*/
	void lineDDAScanLine(QPointF begin, QPointF end,
	                     QColor firstColor, QColor secondColor,
	                     ScanLineTable & table, bool palletize,
	                     bool randomize);

	/// Interpolates colors between two points
	QColor interpolateColors(QColor firstColor, QColor secondColor,
	                         int begin, int end, int position,
	                         bool palletize, bool randomize);

	/// Palletizes a color
	QColor palletizeColor(int r, int g, int b, bool randomize);

	/// The main window form
	Ui::MainWindow mainWindow;
	/// The image we will be painting
	QImage image;
	/// The upper left position of image
	QPoint imagePosition;
	/// The width of the image
	int imageWidth;
	/// The height of the image
	int imageHeight;
	/// The x coordinate center of the image
	int centerX;
	/// The y coordinate center of the image
	int centerY;

	/// The mouse inputs
	QPoint firstPoint;
	QPoint secondPoint;

	/// We need to know if we already have mouse inputs
	bool mouseInputExists;

	/// A seed for the random number generator
	int seed;

}; // Practico1
#endif
