/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SCANLINETABLE
#define SCANLINETABLE

#include <QList>
#include <QColor>

struct scanLineTableItem
{
	int minimumXCoordinate;
	int maximumXCoordinate;
	QColor minColor;
	QColor maxColor;
	bool draw;
};

class ScanLineTable
{
public:
	/// The constructor
	/**
		Here we pass the height of the image, in order to know the span of the Y
		coordinate.
	*/
	ScanLineTable(int imageHeight);

	/// Destructor
	/**
		Here we delete all the pointers to ScanLineTableItem instances.
	*/
	~ScanLineTable();

	/// Add a new item to the list
	void addItem(int xCoordinate, int yCoordinate, QColor color);

	/// Get the minimum and maximum X coordinate for a given Y coordinate
	void getXMinMax(int y, int & minX, QColor & minColor,
	                int & maxX, QColor & maxColor);


private:
	/// The list of points
	QList< struct scanLineTableItem * > table;
	/// The offset
	int offset;
};

#endif

